/*
 * Gimbal
 * Copyright (C) 2024 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.gimbal.mixin;

import net.minecraft.block.BlockState;
import net.minecraft.item.BlockItem;
import net.minecraft.item.Item;
import net.minecraft.item.ItemPlacementContext;
import net.minecraft.state.property.Property;
import net.moonleay.gimbal.editor.ServerEditorManager;
import net.moonleay.gimbal.editor.state.mode.Capability;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Redirect;

import java.util.UUID;

@Mixin(BlockItem.class)
public abstract class ReplaceStateUpdaterMixin extends Item {

    public ReplaceStateUpdaterMixin(Settings settings) {
        super(settings);
    }

    @Shadow
    protected abstract boolean place(ItemPlacementContext context, BlockState state);

    @Redirect(method = "place(Lnet/minecraft/item/ItemPlacementContext;)Lnet/minecraft/util/ActionResult;", at = @At(value = "INVOKE", target = "Lnet/minecraft/item/BlockItem;place(Lnet/minecraft/item/ItemPlacementContext;Lnet/minecraft/block/BlockState;)Z"))
    private boolean func(BlockItem instance, ItemPlacementContext context, BlockState state) {
        if (context.getPlayer() == null)
            return this.place(context, state);
        UUID id = context.getPlayer().getGameProfile().getId();
        if (!ServerEditorManager.INSTANCE.shouldPlayer(id, Capability.REPLACE))
            return this.place(context, state);
        BlockState oldState = context.getWorld().getBlockState(context.getBlockPos());
        return this.place(context, getTargetBlockState(oldState, instance.getBlock().getDefaultState()));
    }

    @Unique
    public BlockState getTargetBlockState(BlockState oldState, BlockState newBlock) {
        var oldManager = oldState.getBlock().getStateManager();
        var newManager = newBlock.getBlock().getStateManager();
        for (var prop : oldManager.getProperties()) {
            var matchingProp = newManager.getProperty(prop.getName());
            if (matchingProp != null) {
                //noinspection rawtypes,unchecked
                newBlock = newBlock.with((Property) matchingProp, oldState.get(matchingProp));
            }
        }
        return newBlock;
    }
}
